<?php
/**
 * Landofcoder
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Landofcoder.com license that is
 * available through the world-wide-web at this URL:
 * http://landofcoder.com/license
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to newer
 * version in the future.
 *
 * @category   Landofcoder
 * @package    Lof_SocialLogin
 *
 * @copyright  Copyright (c) 2016 Landofcoder (http://www.landofcoder.com/)
 * @license    http://www.landofcoder.com/LICENSE-1.0.html
 */

namespace Lof\SocialLogin\Controller\Google;

use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\View\Result\PageFactory;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Customer\Api\AccountManagementInterface;
use Magento\Customer\Model\Url as CustomerUrl;
use Magento\Customer\Model\Session;
use Magento\Customer\Model\CustomerFactory;
use Symfony\Component\Config\Definition\Exception\Exception;
use Lof\SocialLogin\Model\ResourceModel\Social\CollectionFactory as GoogleCollectionFactory;
use Lof\SocialLogin\Model\Google;
use Lof\SocialLogin\Helper\Google\Data as SocialHelper;
use Lof\SocialLogin\Model\SocialFactory as GoogleModelFactory;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Customer\Model\Visitor;

class Callback extends Action
{
    const SOCIAL_TYPE = 'google';
    protected $resultPageFactory;
    protected $google;
    protected $socialHelper;
    protected $accountManagement;
    protected $customerUrl;
    protected $session;
    protected $googleCustomerCollectionFactory;
    protected $googleCustomerModelFactory;
    protected $customerFactory;

    /**
     * @var \Magento\Framework\Stdlib\Cookie\CookieMetadataFactory
     */
    private $cookieMetadataFactory;

    /**
     * @var \Magento\Framework\Stdlib\Cookie\PhpCookieManager
     */
    private $cookieMetadataManager;

    public function __construct(
        Context $context,
        Google $google,
        StoreManagerInterface $storeManager,
        SocialHelper $socialHelper,
        PageFactory $resultPageFactory,
        AccountManagementInterface $accountManagement,
        CustomerUrl $customerUrl,
        GoogleModelFactory $googleCustomerModelFactory,
        GoogleCollectionFactory $googleCustomerCollectionFactory,
        CustomerFactory $customerFactory,
        CustomerRepositoryInterface $customerRepository,
        Session $customerSession,
        Visitor $visitor
        ) {
        parent::__construct($context);
        $this->google                          = $google;
        $this->storeManager                    = $storeManager;
        $this->socialHelper                    = $socialHelper;
        $this->resultPageFactory               = $resultPageFactory;
        $this->accountManagement               = $accountManagement;
        $this->customerUrl                     = $customerUrl;
        $this->session                         = $customerSession;
        $this->googleCustomerModelFactory      = $googleCustomerModelFactory;
        $this->googleCustomerCollectionFactory = $googleCustomerCollectionFactory;
        $this->customerFactory                 = $customerFactory;
        $this->customerRepository              = $customerRepository;
        $this->visitor                         = $visitor;
    }

    public function execute()
    {
        $oauth2 = new \Google_Oauth2Service($this->google);
        $code   = $this->getRequest()->getParam('code');
        if (!$code) {
            echo "<script>window.close()</script>";
            exit;
        }
        $this->google->authenticate($code);
        $dataUser = $oauth2->userinfo->get();
        $redirect = $this->socialHelper->getConfig(('general/redirect_page'));
        if(empty($redirect)){
            $link_redirect = "window.opener.location.reload();";
        }else{
            $link_redirect = "window.opener.location= '".$redirect."';";
        };  
        $data = [];
        $customerIds = $this->getCustomerIdByGoogleId($dataUser['id']);

        $customer = null;
        $customerId = null;
        $storeId = $this->storeManager->getStore()->getId();

        foreach ($customerIds as $id) {
            $customer = $this->customerRepository->getById($id);

            if ($customer->getStoreId() != $storeId) {
                $customerId = null;
                $customer = null;
                continue;
            }

            $customerId = $id;

            break;
        }
        if ($customerId) {
            $customer1 = $this->customerFactory->create()->load($customerId);
            if ($customer->getConfirmation()) {
                try {
                    $customer1->setConfirmation(null);
                    $customer1->save();
                } catch (\Exception $e) {
                    $this->messageManager->addError(__('We can\'t process your request right now. Sorry, that\'s all we know.'));
                }
            }
            if ($this->getCookieManager()->getCookie('mage-cache-sessid')) {
                $metadata = $this->getCookieMetadataFactory()->createCookieMetadata();
                $metadata->setPath('/');
                $this->getCookieManager()->deleteCookie('mage-cache-sessid', $metadata);
            }
            $this->session->setCustomerDataAsLoggedIn($customer);
            $this->messageManager->addSuccess(__('Login successful.'));
            $this->session->regenerateId();
            $this->session->setCustomerDataAsLoggedIn($customer);
            $this->_eventManager->dispatch('customer_data_object_login', ['customer' => $customer]);
            $this->_eventManager->dispatch('customer_login', ['customer' => $customer1]);

            /** VISITOR */
            $visitor = $this->visitor;
            $visitor->setData($this->session->getVisitorData());
            $visitor->setLastVisitAt((new \DateTime())->format(\Magento\Framework\Stdlib\DateTime::DATETIME_PHP_FORMAT));
            $visitor->setSessionId($this->session->getSessionId());
            $visitor->save();
            $this->_eventManager->dispatch('visitor_init', ['visitor' => $visitor]);
            $this->_eventManager->dispatch('visitor_activity_save', ['visitor' => $visitor]);
            echo "<script type=\"text/javascript\">window.close();".$link_redirect."</script>"; 
            exit;
        }
        if ($dataUser) {
            $data['id']                    = $dataUser['id'];
            $data['email']                 = $dataUser['email'];
            $arrName                       = explode(' ', $dataUser['name'], 2);
            $data['password']              =  $this->socialHelper->createPassword();
            $data['password_confirmation'] = $data['password'];
            $data['first_name']            = $arrName[0];
            $data['last_name']             = $arrName[1];

            $store_id   = $this->storeManager->getStore()->getStoreId();
            $website_id = $this->storeManager->getStore()->getWebsiteId();
            $customer   = $this->socialHelper->getCustomerByEmail($data['email'], $website_id);
            if (!$customer || !$customer->getId()) {
                $customer = $this->socialHelper->createCustomerMultiWebsite($data, $website_id, $store_id);
                if ($this->socialHelper->sendPassword()) {
                    try {
                        $customer->sendPasswordReminderEmail();
                    } catch (Exception $e) {
                    }
                }
            }
            $this->setAuthorCustomer($data['id'], $customer->getId());
            $confirmationStatus = $this->accountManagement->getConfirmationStatus($customer->getId());
            if ($confirmationStatus === AccountManagementInterface::ACCOUNT_CONFIRMATION_REQUIRED) {
                $this->customerUrl->getEmailConfirmationUrl($customer->getEmail());
            } else {
                $this->session->setCustomerDataAsLoggedIn($customer);
                $this->messageManager->addSuccess(__('Login successful.'));
                $this->session->regenerateId();
            }
            echo "<script type=\"text/javascript\">window.close();".$link_redirect."</script>"; 
        }
    }

    public function getCustomerIdByGoogleId($googleId)
    {
        $customer = $this->googleCustomerCollectionFactory->create();
        $dataUser     = $customer
            ->addFieldToFilter('social_id', $googleId)
            ->addFieldToFilter('type', self::SOCIAL_TYPE);
        if ($dataUser) {
            $ids = [];
            foreach ($dataUser->getItems() as $user) {
                if (!$user->getId()) {
                    continue;
                }
                $ids[] = $user->getCustomerId();
            }
            if (empty($ids)) {
                return null;
            }
            return $ids;
        } else {
            return null;
        }
    }

    public function setAuthorCustomer($googleId, $customerId)
    {
        $googleCustomer = $this->googleCustomerModelFactory->create();
        $googleCustomer->setData('social_id', $googleId);
        $googleCustomer->setData('username', $googleId);
        $googleCustomer->setData('customer_id', $customerId);
        $googleCustomer->setData('type', self::SOCIAL_TYPE);
        $googleCustomer->setData('is_send_password_email', $this->socialHelper->sendPassword());
        try {
            $googleCustomer->save();
        } catch (Exception $e) {
        }

        return;
    }

    /**
     * Retrieve cookie manager
     *
     * @deprecated
     * @return \Magento\Framework\Stdlib\Cookie\PhpCookieManager
     */
    private function getCookieManager()
    {
        if (!$this->cookieMetadataManager) {
            $this->cookieMetadataManager = \Magento\Framework\App\ObjectManager::getInstance()->get(
                \Magento\Framework\Stdlib\Cookie\PhpCookieManager::class
                );
        }
        return $this->cookieMetadataManager;
    }

    /**
     * Retrieve cookie metadata factory
     *
     * @deprecated
     * @return \Magento\Framework\Stdlib\Cookie\CookieMetadataFactory
     */
    private function getCookieMetadataFactory()
    {
        if (!$this->cookieMetadataFactory) {
            $this->cookieMetadataFactory = \Magento\Framework\App\ObjectManager::getInstance()->get(
                \Magento\Framework\Stdlib\Cookie\CookieMetadataFactory::class
                );
        }
        return $this->cookieMetadataFactory;
    }

}
